searchNodes=[{"ref":"Mine.html","title":"Mine","type":"module","doc":""},{"ref":"Mine.html#__set_default_view__/2","title":"Mine.__set_default_view__/2","type":"function","doc":"Sets the @mine_default_view attribute for mod."},{"ref":"Mine.html#add_field/2","title":"Mine.add_field/2","type":"macro","doc":""},{"ref":"Mine.html#alias_field/2","title":"Mine.alias_field/2","type":"macro","doc":""},{"ref":"Mine.html#default_view/1","title":"Mine.default_view/1","type":"macro","doc":""},{"ref":"Mine.html#defview/2","title":"Mine.defview/2","type":"macro","doc":""},{"ref":"Mine.html#finalize_view/2","title":"Mine.finalize_view/2","type":"function","doc":"Verifies that the declared default key corresponds to an existing view. Raises if this condition fails."},{"ref":"Mine.html#get_view!/3","title":"Mine.get_view!/3","type":"function","doc":""},{"ref":"Mine.html#has_struct?/1","title":"Mine.has_struct?/1","type":"function","doc":"Checks if mod has declared the functions required to be a struct at compile time."},{"ref":"Mine.html#ignore_field/1","title":"Mine.ignore_field/1","type":"macro","doc":""},{"ref":"Mine.html#valid_key?/1","title":"Mine.valid_key?/1","type":"function","doc":""},{"ref":"Mine.html#t:key/0","title":"Mine.key/0","type":"type","doc":""},{"ref":"Mine.Alias.html","title":"Mine.Alias","type":"module","doc":"Stores the data associated with an aliased field. Composed at compile time and used during runtime to determine how a field should be translated."},{"ref":"Mine.Alias.html#merge/3","title":"Mine.Alias.merge/3","type":"function","doc":"Utility function for merging two Mine.Alias structs. Currently only returns the second struct."},{"ref":"Mine.Alias.html#new/2","title":"Mine.Alias.new/2","type":"function","doc":"Creates a new Mine.Alias struct"},{"ref":"Mine.Alias.html#validate/1","title":"Mine.Alias.validate/1","type":"function","doc":""},{"ref":"Mine.Alias.html#t:t/0","title":"Mine.Alias.t/0","type":"type","doc":""},{"ref":"Mine.Builder.html","title":"Mine.Builder","type":"module","doc":""},{"ref":"Mine.Builder.html#build_from_view/3","title":"Mine.Builder.build_from_view/3","type":"function","doc":""},{"ref":"Mine.Builder.html#build_to_view/3","title":"Mine.Builder.build_to_view/3","type":"function","doc":""},{"ref":"Mine.View.CompileError.html","title":"Mine.View.CompileError","type":"exception","doc":""},{"ref":"Mine.View.CompileError.html#t:t/0","title":"Mine.View.CompileError.t/0","type":"type","doc":""},{"ref":"readme.html","title":"Mine","type":"extras","doc":"Mine Mine is a lightweight package for defining views for structs. It aims to eliminate the boilerplate required to write and maintain an anti-corruption layer between your structs and any external APIs you interface with. Note: Mine currently exports the minimum functionality required for my own use case but is not ready for adoption yet. Until v1.0.0, the interface may change in ways that break backwards compatibility."},{"ref":"readme.html#usage","title":"Mine - Usage","type":"extras","doc":"Basic Views defmodule User do use Mine # module must declare a struct defstruct [:name, :email, :password] defview do alias_field :name, as: &quot;userName&quot;, default: &quot;John Doe&quot; ignore_field :password end end A view is created by importing Mine at the beginning of your module and using the exported defview/2 macro. Note that User declares a struct and the defview is used after the declaration. This requirement stems from the intent of Mine to be purely additive. It will not replace functionality provided by more mature modules but rather build on existing code. The most notable benefit to this approach is that other packages that internally define structs (like Ecto) can be extended. At compile time, Mine will create views for every module in use. Any incompatibilities between the declared views and their base structs will be caught here, providing the benefit of runtime safety. To access the compiled view at runtime, the following methods will be found on your module. to_view/2: Covert struct to a map for a given view from_view/2: Convert source map to struct. Convenience method for normalizing an input map, using it to create a struct, and validating the result. Relies on the above methods to accomplish this. These functions should ideally be used between your serialization layer and business logic. More info on these functions can be found on HexDocs. Available Macros There are several macros that can be used inside the scope of defview. They are: alias_field/2: Change the key for a given field. Has the following uses: alias_field(:key, &quot;as&quot;) View will use &quot;as&quot; instead of the field name :key alias_field(:key, default: &quot;def&quot;) Same as above, but if :key is not found or it&#39;s value is nil, &quot;def&quot; will be used as the value instead. alias_field(:key, map_to: &amp;String.upcase/1) When using to_view, value of field in struct will be accessed, mapped using map_to, and stored under the key &quot;as&quot; in the resulting map alias_field(:key, map_from: &amp;String.downcase/1) When using from_view, value of &quot;as&quot; in the given map will be fetched and passed as the only argument to the function in map_from. ignore_field/1: Field will be ignored in both to_view and from from_view. add_field/2: A key value pair that will be added to any map produced by to_view. Ignored in from_view. Named Views In some cases, it may not be enough to have a single exported view of a struct. To support this use case, every view created by Mine has a name (with :default) being the default. The additional functions to_view/1 and from_view/1 work with the default view name. A more complicated User module may declare views such as these: ... # functions with arity 1 will use this view default_view :front_end # other fields will remain untouched defview :third_party_api do alias_field :name, as: &quot;userName&quot; end defview :front_end do alias_field :name, default: &quot;?&quot; alias_field :email, default: &quot;unknown&quot; ignore_field :password end ..."},{"ref":"readme.html#rationale","title":"Mine - Rationale","type":"extras","doc":"While interfacing with an external API written in Java, I frequently ran across instances in which I would need to map a struct&#39;s key, ignore certain fields, or add constant fields to outgoing requests. The process of setting up these mappings is tiresome and potentially error prone. Let&#39;s take the most extreme example I encountered. This is the required JSON format for a port in an unnamed API: { &quot;$&quot;: 7000, &quot;@enabled&quot;: false } Peculiar formats such as this were littered throughout the API, leading to several modules with structures similar to the following: defmodule PortV1 do defstruct [:num, :enabled] def to_view(%PortV1{num: num, enabled: enabled}) do %{ &quot;$&quot; =&gt; num, &quot;@enabled&quot; =&gt; enabled } end def from_view(%{&quot;$&quot; =&gt; num, &quot;@enabled&quot; =&gt; enabled}) do %PortV1{num: num, enabled: enabled} end end Code like this: is necessary to maintain a clean internal structure adds more noise than meaning can be error prone Given these conditions, I opted to break my first rule of writing macros in Elixir (avoid it). Instead of the example seen above, using mine allows for a much more concise representation of a mapping to the external world. defmodule PortV2 do use Mine defstruct [:num, :enabled] defview do alias_field :num, &quot;$&quot; alias_field :enabled, &quot;@enabled&quot; end end This representation is easier to read, maintain, and is even checked for validity at compile time. In addition, the generated code is nearly identical to the code written by hand, resulting in a minimal performance impact."},{"ref":"readme.html#installation","title":"Mine - Installation","type":"extras","doc":"Mine can be installed by adding mine to your list of dependencies in mix.exs: def deps do [ {:mine, &quot;~&gt; 0.2.1&quot;} ] end"},{"ref":"readme.html#benchmarking","title":"Mine - Benchmarking","type":"extras","doc":"To benchmark the generated functions against those written by hand, run the following: shell script MIX_ENV=bench mix run bench/run.exs"},{"ref":"readme.html#roadmap","title":"Mine - Roadmap","type":"extras","doc":"If this project gathers any interest, here are some steps I would like to take to improve the library: [ ] Compare mapping strategies to determine the fastest approach [ ] Additional options for when to use default values [ ] embedded_view key, module macro: signals that the contents of the target field is also a struct using Mine and should be translated accordingly in to_view and from_view [ ] Ecto integration: when a given module is using Ecto, validate will automatically use the changeset/1 function for validation [ ] Phoenix integration via Plug: simple plug to map request/response bodies to their intended form after parsing"},{"ref":"changelog.html","title":"Changelog","type":"extras","doc":"Changelog"},{"ref":"changelog.html#v0-2-1","title":"Changelog - v0.2.1","type":"extras","doc":"New Feature: Mapping Functions map_to and map_from. When an anonymous (or captured) function with arity/1 is supplied to alias_field using the mentioned key, values will be mapped after translation. # Example module using mapping feature defmodule A do use Mine defstruct [:b] defview do alias_field :b, as: &quot;B&quot;, map_to: &amp;String.upcase/1 end end Functions given to map_to and map_from maintain the scope of the caller. Private functions and aliases are honored. Imported functions are currently not supported. Values supplied as mappers are trivially validated to be functions. Improvements Macro hygiene. alias_field, ignore_field, etc are now scoped to defview. If as option is not passed to alias_field, the key name will be turned into a string and used instead. Fixed edge case where aliasing after a defview would complain because the view_agent was still registered but not alive. # example defmodule C do use Mine # defines struct defview do # body end # would randomly cause a crash ignore_field(:some_field) end"},{"ref":"from_view_0-2-0_report.html","title":"Benchmark - from_view","type":"extras","doc":"Benchmark - from_view Benchmark run from 2020-01-17 19:15:53.695741Z UTC"},{"ref":"from_view_0-2-0_report.html#system","title":"Benchmark - from_view - System","type":"extras","doc":"Benchmark suite executing on the following system: Operating System Linux CPU Information Intel(R) Core(TM) i7-6500U CPU @ 2.50GHz Number of Available Cores 4 Available Memory 7.70 GB Elixir Version 1.9.4 Erlang Version 22.2"},{"ref":"from_view_0-2-0_report.html#configuration","title":"Benchmark - from_view - Configuration","type":"extras","doc":"Benchmark suite executing with the following configuration: :time 2 s :parallel 1 :warmup 2 s"},{"ref":"from_view_0-2-0_report.html#statistics","title":"Benchmark - from_view - Statistics","type":"extras","doc":"Input: empty map Run Time Name IPS Average Devitation Median 99th&nbsp;% Elixir - Functions with Arity >1 20.63 M 48.47 ns ±23011.81% 0 ns 333.50 ns Mine 19.02 M 52.57 ns ±33319.80% 0 ns 170.50 ns Elixir - Functions with Arity 1 16.46 M 60.74 ns ±28104.70% 0 ns 180.50 ns Comparison Name IPS Slower Elixir - Functions with Arity >1 20.63 M &nbsp; Mine 19.02 M 1.08x Elixir - Functions with Arity 1 16.46 M 1.25x Memory Usage Name Memory Factor Elixir - Functions with Arity >1 48 B &nbsp; Mine 48 B 1.0x Elixir - Functions with Arity 1 48 B 1.0x Input: expected Run Time Name IPS Average Devitation Median 99th&nbsp;% Mine 18.27 M 54.74 ns ±27935.78% 0 ns 406.50 ns Elixir - Functions with Arity 1 16.55 M 60.42 ns ±22270.10% 0 ns 268.50 ns Elixir - Functions with Arity >1 10.22 M 97.89 ns ±12314.83% 0 ns 588.50 ns Comparison Name IPS Slower Mine 18.27 M &nbsp; Elixir - Functions with Arity 1 16.55 M 1.1x Elixir - Functions with Arity >1 10.22 M 1.79x Memory Usage Name Memory Factor Mine 48 B &nbsp; Elixir - Functions with Arity 1 48 B 1.0x Elixir - Functions with Arity >1 48 B 1.0x"},{"ref":"to_view_0-2-0_report.html","title":"Benchmark - to_view","type":"extras","doc":"Benchmark - to_view Benchmark run from 2020-01-17 19:15:08.455840Z UTC"},{"ref":"to_view_0-2-0_report.html#system","title":"Benchmark - to_view - System","type":"extras","doc":"Benchmark suite executing on the following system: Operating System Linux CPU Information Intel(R) Core(TM) i7-6500U CPU @ 2.50GHz Number of Available Cores 4 Available Memory 7.70 GB Elixir Version 1.9.4 Erlang Version 22.2"},{"ref":"to_view_0-2-0_report.html#configuration","title":"Benchmark - to_view - Configuration","type":"extras","doc":"Benchmark suite executing with the following configuration: :time 2 s :parallel 1 :warmup 2 s"},{"ref":"to_view_0-2-0_report.html#statistics","title":"Benchmark - to_view - Statistics","type":"extras","doc":"Input: empty Run Time Name IPS Average Devitation Median 99th&nbsp;% Elixir - Functions with Arity 1 76.48 M 13.07 ns ±28689.35% 0 ns 0 ns Mine 74.43 M 13.44 ns ±24194.71% 0 ns 6 ns Elixir - Functions with Arity >1 68.92 M 14.51 ns ±24759.87% 0 ns 126 ns Comparison Name IPS Slower Elixir - Functions with Arity 1 76.48 M &nbsp; Mine 74.43 M 1.03x Elixir - Functions with Arity >1 68.92 M 1.11x Memory Usage Name Memory Factor Elixir - Functions with Arity 1 40 B &nbsp; Mine 40 B 1.0x Elixir - Functions with Arity >1 40 B 1.0x Input: expected Run Time Name IPS Average Devitation Median 99th&nbsp;% Mine 27.45 M 36.43 ns ±13258.40% 0 ns 556 ns Elixir - Functions with Arity 1 17.31 M 57.78 ns ±7814.67% 0 ns 996 ns Elixir - Functions with Arity >1 16.07 M 62.22 ns ±8013.65% 0 ns 669 ns Comparison Name IPS Slower Mine 27.45 M &nbsp; Elixir - Functions with Arity 1 17.31 M 1.59x Elixir - Functions with Arity >1 16.07 M 1.71x Memory Usage Name Memory Factor Mine 136 B &nbsp; Elixir - Functions with Arity 1 136 B 1.0x Elixir - Functions with Arity >1 136 B 1.0x"}]